@@ -1,20 +1,15 @@
 import os
 import shutil
+import tempfile
 from contextlib import contextmanager
-from distutils.dir_util import copy_tree
 from pathlib import Path
 from tempfile import mkdtemp
-from typing import List
 
 import ape
 import pytest
 import vvm  # type: ignore
-
-# NOTE: Ensure that we don't use local paths for these
-DATA_FOLDER = Path(mkdtemp()).resolve()
-PROJECT_FOLDER = Path(mkdtemp()).resolve()
-ape.config.DATA_FOLDER = DATA_FOLDER
-ape.config.PROJECT_FOLDER = PROJECT_FOLDER
+from ape.contracts import ContractContainer
+from click.testing import CliRunner
 
 BASE_CONTRACTS_PATH = Path(__file__).parent / "contracts"
 TEMPLATES_PATH = BASE_CONTRACTS_PATH / "templates"
@@ -34,6 +29,7 @@ ALL_VERSIONS = (
     "0.3.7",
     "0.3.9",
     "0.3.10",
+    "0.4.0rc6",
 )
 
 CONTRACT_VERSION_GEN_MAP = {
@@ -42,11 +38,36 @@ CONTRACT_VERSION_GEN_MAP = {
         "0.3.9",
         "0.3.10",
     ),
-    "sub_reverts": ALL_VERSIONS,
+    "sub_reverts": [v for v in ALL_VERSIONS if "0.4.0" not in v],
 }
 
 
-def contract_test_cases(passing: bool) -> List[str]:
+@pytest.fixture(scope="session", autouse=True)
+def from_tests_dir():
+    # Makes default project correct.
+    here = Path(__file__).parent
+    orig = Path.cwd()
+    if orig != here:
+        os.chdir(f"{here}")
+
+    yield
+
+    if Path.cwd() != orig:
+        os.chdir(f"{orig}")
+
+
+@pytest.fixture(scope="session", autouse=True)
+def config():
+    cfg = ape.config
+
+    # Ensure we don't persist any .ape data.
+    with tempfile.TemporaryDirectory() as temp_dir:
+        path = Path(temp_dir).resolve()
+        cfg.DATA_FOLDER = path
+        yield cfg
+
+
+def contract_test_cases(passing: bool) -> list[str]:
     """
     Returns test-case names for outputting nicely with pytest.
     """
@@ -126,16 +147,6 @@ def temp_vvm_path(monkeypatch):
         yield path
 
 
-@pytest.fixture
-def data_folder():
-    return DATA_FOLDER
-
-
-@pytest.fixture
-def project_folder():
-    return PROJECT_FOLDER
-
-
 @pytest.fixture
 def compiler_manager():
     return ape.compilers
@@ -146,35 +157,24 @@ def compiler(compiler_manager):
     return compiler_manager.vyper
 
 
-@pytest.fixture
-def config():
-    return ape.config
-
-
-@pytest.fixture(autouse=True)
+@pytest.fixture(scope="session", autouse=True)
 def project(config):
     project_source_dir = Path(__file__).parent
-    project_dest_dir = config.PROJECT_FOLDER / project_source_dir.name
 
     # Delete build / .cache that may exist pre-copy
-    project_path = Path(__file__).parent
-    cache = project_path / ".build"
+    cache = project_source_dir / ".build"
+    shutil.rmtree(cache, ignore_errors=True)
 
-    if cache.is_dir():
-        shutil.rmtree(cache)
-
-    copy_tree(project_source_dir.as_posix(), project_dest_dir.as_posix())
-    with config.using_project(project_dest_dir) as project:
-        yield project
-        if project.local_project._cache_folder.is_dir():
-            shutil.rmtree(project.local_project._cache_folder)
+    root_project = ape.Project(project_source_dir)
+    with root_project.isolate_in_tempdir() as tmp_project:
+        yield tmp_project
 
 
 @pytest.fixture
 def geth_provider():
-    if not ape.networks.active_provider or ape.networks.provider.name != "geth":
+    if not ape.networks.active_provider or ape.networks.provider.name != "node":
         with ape.networks.ethereum.local.use_provider(
-            "geth", provider_settings={"uri": "http://127.0.0.1:5550"}
+            "node", provider_settings={"uri": "http://127.0.0.1:5550"}
         ) as provider:
             yield provider
     else:
@@ -211,8 +211,17 @@ def all_versions():
     return ALL_VERSIONS
 
 
+@pytest.fixture
+def cli_runner():
+    return CliRunner()
+
+
 def _get_tb_contract(version: str, project, account):
+    project.load_contracts()
+
     registry_type = project.get_contract(f"registry_{version}")
+    assert isinstance(registry_type, ContractContainer), "Setup failed - couldn't get container"
     registry = account.deploy(registry_type)
     contract = project.get_contract(f"traceback_contract_{version}")
+    assert isinstance(contract, ContractContainer), "Setup failed - couldn't get container"
     return account.deploy(contract, registry)
