@@ -1,10 +1,13 @@
 import re
+from pathlib import Path
 
+import ape
 import pytest
-from ape.exceptions import ContractLogicError
+import vvm  # type: ignore
+from ape.exceptions import CompilerError, ContractLogicError
+from ape.utils import get_full_extension
 from ethpm_types import ContractType
 from packaging.version import Version
-from vvm import compile_source  # type: ignore
 from vvm.exceptions import VyperError  # type: ignore
 
 from ape_vyper.compiler import RuntimeErrorType
@@ -23,6 +26,7 @@ from .conftest import FAILING_BASE, FAILING_CONTRACT_NAMES, PASSING_CONTRACT_NAM
 OLDER_VERSION_FROM_PRAGMA = Version("0.2.16")
 VERSION_37 = Version("0.3.7")
 VERSION_FROM_PRAGMA = Version("0.3.10")
+VERSION_04 = Version("0.4.0rc6")
 
 
 @pytest.fixture
@@ -44,43 +48,60 @@ EXPECTED_FAIL_PATTERNS = {
 
 
 def test_compile_project(project):
-    contracts = project.load_contracts()
-    assert len(contracts) == len(
-        [p.name for p in project.contracts_folder.glob("*.vy") if p.is_file()]
+    actual = sorted(list(project.load_contracts().keys()))
+
+    # NOTE: Ignore interfaces for this test.
+    expected = sorted(
+        [
+            p.stem
+            for p in project.contracts_folder.rglob("*.vy")
+            if p.is_file() and not p.name.startswith("I")
+        ]
     )
-    assert contracts["contract_039"].source_id == "contract_039.vy"
-    assert contracts["contract_no_pragma"].source_id == "contract_no_pragma.vy"
-    assert contracts["older_version"].source_id == "older_version.vy"
+    if missing := [e for e in expected if e not in actual]:
+        missing_str = ", ".join(missing)
+        pytest.xfail(f"Missing the following expected sources: {missing_str}")
+    if extra := [a for a in actual if a not in expected]:
+        extra_str = ", ".join(extra)
+        pytest.xfail(f"Received the following extra sources: {extra_str}")
+
+    assert "contract_039" in actual
+    assert "contract_no_pragma" in actual
+    assert "older_version" in actual
 
 
 @pytest.mark.parametrize("contract_name", PASSING_CONTRACT_NAMES)
 def test_compile_individual_contracts(project, contract_name, compiler):
     path = project.contracts_folder / contract_name
-    assert compiler.compile([path])
+    assert list(compiler.compile((path,), project=project))
 
 
 @pytest.mark.parametrize(
     "contract_name", [n for n in FAILING_CONTRACT_NAMES if n != "contract_unknown_pragma.vy"]
 )
 def test_compile_failures(contract_name, compiler):
+    failing_project = ape.Project(FAILING_BASE)
     path = FAILING_BASE / contract_name
     with pytest.raises(VyperCompileError, match=EXPECTED_FAIL_PATTERNS[path.stem]) as err:
-        compiler.compile([path], base_path=FAILING_BASE)
+        list(compiler.compile((path,), project=failing_project))
 
     assert isinstance(err.value.base_err, VyperError)
 
 
 def test_install_failure(compiler):
+    failing_project = ape.Project(FAILING_BASE)
     path = FAILING_BASE / "contract_unknown_pragma.vy"
     with pytest.raises(VyperInstallError, match="No available version to install."):
-        compiler.compile([path])
+        list(compiler.compile((path,), project=failing_project))
 
 
 def test_get_version_map(project, compiler, all_versions):
     vyper_files = [
-        x for x in project.contracts_folder.iterdir() if x.is_file() and x.suffix == ".vy"
+        x
+        for x in project.contracts_folder.iterdir()
+        if x.is_file() and get_full_extension(x) == ".vy"
     ]
-    actual = compiler.get_version_map(vyper_files)
+    actual = compiler.get_version_map(vyper_files, project=project)
     expected_versions = [Version(v) for v in all_versions]
 
     for version, sources in actual.items():
@@ -100,10 +121,12 @@ def test_get_version_map(project, compiler, all_versions):
         "erc20.vy",
         "use_iface.vy",
         "optimize_codesize.vy",
+        "evm_pragma.vy",
         "use_iface2.vy",
         "contract_no_pragma.vy",  # no pragma should compile with latest version
         "empty.vy",  # empty file still compiles with latest version
         "pragma_with_space.vy",
+        "flatten_me.vy",
     ]
 
     # Add the 0.3.10 contracts.
@@ -130,27 +153,55 @@ def test_get_version_map(project, compiler, all_versions):
 
     assert not failures, "\n".join(failures)
 
+    # Vyper 0.4.0 assertions.
+    actual4 = {x.name for x in actual[VERSION_04]}
+    expected4 = {"zero_four_module.vy", "zero_four.vy"}
+    assert actual4 == expected4
+
 
 def test_compiler_data_in_manifest(project):
-    _ = project.contracts
-    manifest = project.extract_manifest()
-    assert len(manifest.compilers) >= 3, manifest.compilers
+    def run_test(manifest):
+        assert len(manifest.compilers) >= 3, manifest.compilers
 
-    vyper_latest = [c for c in manifest.compilers if str(c.version) == str(VERSION_FROM_PRAGMA)][0]
-    vyper_028 = [c for c in manifest.compilers if str(c.version) == str(OLDER_VERSION_FROM_PRAGMA)][
-        0
-    ]
+        all_latest = [c for c in manifest.compilers if str(c.version) == str(VERSION_FROM_PRAGMA)]
+        codesize_latest = [c for c in all_latest if c.settings["optimize"] == "codesize"][0]
+        evm_latest = [c for c in all_latest if c.settings.get("evmVersion") == "paris"][0]
+        true_latest = [
+            c
+            for c in all_latest
+            if c.settings["optimize"] is True and c.settings.get("evmVersion") != "paris"
+        ][0]
+        vyper_028 = [
+            c for c in manifest.compilers if str(c.version) == str(OLDER_VERSION_FROM_PRAGMA)
+        ][0]
 
-    for compiler in (vyper_028, vyper_latest):
-        assert compiler.name == "vyper"
+        for compiler in (vyper_028, codesize_latest, true_latest):
+            assert compiler.name == "vyper"
 
-    assert len(vyper_latest.contractTypes) >= 9
-    assert len(vyper_028.contractTypes) >= 1
-    assert "contract_0310" in vyper_latest.contractTypes
-    assert "older_version" in vyper_028.contractTypes
-    for compiler in (vyper_latest, vyper_028):
-        assert compiler.settings["evmVersion"] == "istanbul"
-        assert compiler.settings["optimize"] is True
+        assert vyper_028.settings["evmVersion"] == "berlin"
+        assert codesize_latest.settings["evmVersion"] == "shanghai"
+        assert true_latest.settings["evmVersion"] == "shanghai"
+
+        # There is only one contract with codesize pragma.
+        assert codesize_latest.contractTypes == ["optimize_codesize"]
+        assert codesize_latest.settings["optimize"] == "codesize"
+
+        # There is only one contract with evm-version pragma.
+        assert evm_latest.contractTypes == ["evm_pragma"]
+        assert evm_latest.settings.get("evmVersion") == "paris"
+
+        assert len(true_latest.contractTypes) >= 9
+        assert len(vyper_028.contractTypes) >= 1
+        assert "contract_0310" in true_latest.contractTypes
+        assert "older_version" in vyper_028.contractTypes
+        for compiler in (true_latest, vyper_028):
+            assert compiler.settings["optimize"] is True
+
+    project.update_manifest(compilers=[])
+    project.load_contracts(use_cache=False)
+    run_test(project.manifest)
+    man = project.extract_manifest()
+    run_test(man)
 
 
 def test_compile_parse_dev_messages(compiler, dev_revert_source, project):
@@ -160,7 +211,7 @@ def test_compile_parse_dev_messages(compiler, dev_revert_source, project):
     The compiler will output a map that maps dev messages to line numbers.
     See contract_with_dev_messages.vy for more information.
     """
-    result = compiler.compile([dev_revert_source], base_path=project.contracts_folder)
+    result = list(compiler.compile((dev_revert_source,), project=project))
 
     assert len(result) == 1
 
@@ -179,18 +230,22 @@ def test_get_imports(compiler, project):
     vyper_files = [
         x for x in project.contracts_folder.iterdir() if x.is_file() and x.suffix == ".vy"
     ]
-    actual = compiler.get_imports(vyper_files)
+    actual = compiler.get_imports(vyper_files, project=project)
+    prefix = "contracts/passing_contracts"
     builtin_import = "vyper/interfaces/ERC20.json"
-    local_import = "interfaces/IFace.vy"
-    local_from_import = "interfaces/IFace2.vy"
-    dependency_import = "exampledep/Dependency.json"
-
-    assert len(actual["contract_037.vy"]) == 1
-    assert set(actual["contract_037.vy"]) == {builtin_import}
-    assert len(actual["use_iface.vy"]) == 3
-    assert set(actual["use_iface.vy"]) == {local_import, local_from_import, dependency_import}
-    assert len(actual["use_iface2.vy"]) == 1
-    assert set(actual["use_iface2.vy"]) == {local_import}
+    local_import = f"{prefix}/interfaces/IFace.vy"
+    local_from_import = f"{prefix}/interfaces/IFace2.vy"
+    dep_key = project.dependencies.get_dependency("exampledependency", "local").package_id.replace(
+        "/", "_"
+    )
+    dependency_import = f"{dep_key}/local/contracts/Dependency.vy"
+    assert set(actual[f"{prefix}/contract_037.vy"]) == {builtin_import}
+    assert set(actual[f"{prefix}/use_iface.vy"]) == {
+        local_import,
+        local_from_import,
+        dependency_import,
+    }
+    assert set(actual[f"{prefix}/use_iface2.vy"]) == {local_import}
 
 
 @pytest.mark.parametrize("src,vers", [("contract_039", "0.3.9"), ("contract_037", "0.3.7")])
@@ -200,14 +255,16 @@ def test_pc_map(compiler, project, src, vers):
     from `compile_src()` which includes the uncompressed source map data.
     """
 
-    path = project.contracts_folder / f"{src}.vy"
-    result = compiler.compile([path], base_path=project.contracts_folder)[0]
-    actual = result.pcmap.__root__
+    path = project.sources.lookup(src)
+    result = list(compiler.compile((path,), project=project))[0]
+    actual = result.pcmap.root
     code = path.read_text()
-    compile_result = compile_source(code, vyper_version=vers, evm_version=compiler.evm_version)[
-        "<stdin>"
-    ]
-    src_map = compile_result["source_map"]
+    vvm.install_vyper(vers)
+    cfg = compiler.get_config(project=project)
+    evm_version = cfg.evm_version
+    compile_result = vvm.compile_source(code, vyper_version=vers, evm_version=evm_version)
+    std_result = compile_result["<stdin>"]
+    src_map = std_result["source_map"]
     lines = code.splitlines()
 
     # Use the old-fashioned way of gathering PCMap to ensure our creative way works
@@ -363,7 +420,7 @@ def test_enrich_error_handle_when_name(compiler, geth_provider, mocker):
 def test_trace_source(account, geth_provider, project, traceback_contract, arguments):
     receipt = traceback_contract.addBalance(*arguments, sender=account)
     actual = receipt.source_traceback
-    base_folder = project.contracts_folder
+    base_folder = Path(__file__).parent / "contracts" / "passing_contracts"
     contract_name = traceback_contract.contract_type.name
     expected = rf"""
 Traceback (most recent call last)
@@ -411,7 +468,7 @@ def test_trace_err_source(account, geth_provider, project, traceback_contract):
 
     receipt = geth_provider.get_receipt(txn.txn_hash.hex())
     actual = receipt.source_traceback
-    base_folder = project.contracts_folder
+    base_folder = Path(__file__).parent / "contracts" / "passing_contracts"
     contract_name = traceback_contract.contract_type.name
     version_key = contract_name.split("traceback_contract_")[-1]
     expected = rf"""
@@ -452,19 +509,20 @@ def test_compile_with_version_set_in_config(config, projects_path, compiler, moc
     path = projects_path / "version_in_config"
     version_from_config = "0.3.7"
     spy = mocker.patch("ape_vyper.compiler.vvm_compile_standard")
-    with config.using_project(path) as project:
-        contract = project.contracts_folder / "v_contract.vy"
-        settings = compiler.get_compiler_settings((contract,))
-        assert str(list(settings.keys())[0]) == version_from_config
+    project = ape.Project(path)
 
-        # Show it uses this version in the compiler.
-        project.load_contracts(use_cache=False)
-        assert str(spy.call_args[1]["vyper_version"]) == version_from_config
+    contract = project.contracts_folder / "v_contract.vy"
+    settings = compiler.get_compiler_settings((contract,), project=project)
+    assert str(list(settings.keys())[0]) == version_from_config
+
+    # Show it uses this version in the compiler.
+    project.load_contracts(use_cache=False)
+    assert str(spy.call_args[1]["vyper_version"]) == version_from_config
 
 
-def test_compile_code(compiler, dev_revert_source):
+def test_compile_code(project, compiler, dev_revert_source):
     code = dev_revert_source.read_text()
-    actual = compiler.compile_code(code, contractName="MyContract")
+    actual = compiler.compile_code(code, project=project, contractName="MyContract")
     assert isinstance(actual, ContractType)
     assert actual.name == "MyContract"
     assert len(actual.abi) > 1
@@ -475,10 +533,36 @@ def test_compile_code(compiler, dev_revert_source):
 def test_compile_with_version_set_in_settings_dict(config, compiler_manager, projects_path):
     path = projects_path / "version_in_config"
     contract = path / "contracts" / "v_contract.vy"
+    project = ape.Project(path)
+    expected = '.*Version specification "0.3.10" is not compatible with compiler version "0.3.3"'
+    iterator = compiler_manager.compile(
+        (contract,), project=project, settings={"vyper": {"version": "0.3.3"}}
+    )
+    with pytest.raises(CompilerError, match=expected):
+        _ = list(iterator)
 
-    with config.using_project(path):
-        expected = (
-            '.*Version specification "0.3.10" is not compatible with compiler version "0.3.3"'
-        )
-        with pytest.raises(VyperCompileError, match=expected):
-            compiler_manager.compile([contract], settings={"version": "0.3.3"})
+
+@pytest.mark.parametrize(
+    "contract_name",
+    [
+        # This first one has most known edge cases
+        "flatten_me.vy",
+        # Test on the below for general compatibility.
+        "contract_with_dev_messages.vy",
+        "erc20.vy",
+        "use_iface.vy",
+        "optimize_codesize.vy",
+        "evm_pragma.vy",
+        "use_iface2.vy",
+        "contract_no_pragma.vy",  # no pragma should compile with latest version
+        "empty.vy",  # empty file still compiles with latest version
+        "pragma_with_space.vy",
+    ],
+)
+def test_flatten_contract(all_versions, project, contract_name, compiler):
+    path = project.contracts_folder / contract_name
+    source = compiler.flatten_contract(path, project=project)
+    source_code = str(source)
+    version = compiler._source_vyper_version(source_code)
+    vvm.install_vyper(str(version))
+    vvm.compile_source(source_code, base_path=project.path, vyper_version=version)
